<?php $root=""; ?>
<?php require($root."navigation.php"); ?>
<html>
<head>
  <?php load_style($root); ?>
</head>
 
<body>
 
<?php make_navigation("ex23",$root)?>
 
<div class="content">
<a name="comments"></a> 
<div class = "comment">
  

<br><br>rbOOmit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.
  

<br><br>You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


<br><br>C++ include files that we need
</div>

<div class ="fragment">
<pre>
        #include &lt;iostream&gt;
        #include &lt;algorithm&gt;
        #include &lt;cmath&gt;
        #include &lt;set&gt;
        
</pre>
</div>
<div class = "comment">
Basic include file needed for the mesh functionality.
</div>

<div class ="fragment">
<pre>
        #include "libmesh.h"
        #include "mesh.h"
        #include "mesh_generation.h"
        #include "exodusII_io.h"
        #include "equation_systems.h"
        #include "dof_map.h"
        #include "getpot.h"
        #include "o_string_stream.h"
        #include "elem.h"
        
        #include "simple_rb.h"
        #include "assembly.h"
        
</pre>
</div>
<div class = "comment">
In this example problem we use the Certified Reduced Basis method
to solve a steady convection-diffusion problem on the unit square.
The reduced basis method relies on an expansion of the PDE in the
form \sum_q=1^Q_a theta_a^q(\mu) * a^q(u,v) = \sum_q=1^Q_f theta_f^q(\mu) f^q(v)
where theta_a, theta_f are parameter dependent functions and 
a^q, f^q are parameter independent operators (\mu denotes a parameter).


<br><br>We first attach the parameter dependent functions and paramater
independent operators to the RBSystem. Then in Offline mode, a
reduced basis space is generated and written out to the directory
"offline_data". In Online mode, the reduced basis data in "offline_data"
is read in and used to solve the reduced problem for the parameters
specified in ex23.in.


<br><br>We also attach four outputs to the system which are averages over certain
subregions of the domain. In Online mode, we print out the values of these
outputs as well as rigorous error bounds with respect to the output
associated with the "truth" finite element discretization.


<br><br>The main program.
</div>

<div class ="fragment">
<pre>
        int main (int argc, char** argv)
        {
</pre>
</div>
<div class = "comment">
Initialize libMesh.
</div>

<div class ="fragment">
<pre>
          LibMeshInit init (argc, argv);
        
        #if !defined(LIBMESH_HAVE_XDR)
</pre>
</div>
<div class = "comment">
We need XDR support to write out reduced bases
</div>

<div class ="fragment">
<pre>
          libmesh_example_assert(false, "--enable-xdr");
        #elif defined(LIBMESH_DEFAULT_SINGLE_PRECISION)
</pre>
</div>
<div class = "comment">
XDR binary support requires double precision
</div>

<div class ="fragment">
<pre>
          libmesh_example_assert(false, "--disable-singleprecision");
        #endif
</pre>
</div>
<div class = "comment">
FIXME: This example currently segfaults with Trilinos?
</div>

<div class ="fragment">
<pre>
          libmesh_example_assert(libMesh::default_solver_package() == PETSC_SOLVERS, "--enable-petsc");
        
</pre>
</div>
<div class = "comment">
Skip this 2D example if libMesh was compiled as 1D-only.
</div>

<div class ="fragment">
<pre>
          libmesh_example_assert(2 &lt;= LIBMESH_DIM, "2D support");
          
</pre>
</div>
<div class = "comment">
Parse the input file (ex23.in) using GetPot
</div>

<div class ="fragment">
<pre>
          std::string parameters_filename = "ex23.in";
          GetPot infile(parameters_filename);
        
          unsigned int n_elem = infile("n_elem", 1);       // Determines the number of elements in the "truth" mesh
          const unsigned int dim = 2;                      // The number of spatial dimensions
          
          bool online_mode = infile("online_mode", false); // Are we in Online mode?
        
</pre>
</div>
<div class = "comment">
Create a one-dimensional mesh.
</div>

<div class ="fragment">
<pre>
          Mesh mesh (dim);
          MeshTools::Generation::build_square (mesh,
                                               n_elem, n_elem,
                                               0., 1.,
                                               0., 1.,
                                               QUAD4);
        
</pre>
</div>
<div class = "comment">
Create an equation systems object.
</div>

<div class ="fragment">
<pre>
          EquationSystems equation_systems (mesh);
        
</pre>
</div>
<div class = "comment">
We override RBSystem with SimpleRB in order to specialize a few functions.
</div>

<div class ="fragment">
<pre>
          SimpleRB & system =
            equation_systems.add_system&lt;SimpleRB&gt; ("RBConvectionDiffusion");
        
        
</pre>
</div>
<div class = "comment">
Initialize the data structures for the equation system.
</div>

<div class ="fragment">
<pre>
          equation_systems.init ();
        
          if(system.initialize_mesh_dependent_data)
          {
</pre>
</div>
<div class = "comment">
Print out some information about the "truth" discretization
</div>

<div class ="fragment">
<pre>
            equation_systems.print_info();
            mesh.print_info();
          }
        
</pre>
</div>
<div class = "comment">
Now that the libMesh data structures have been initialized
in equation_systems.init(), we can set up the Reduced Basis system.
  

<br><br>Point the systems to the input file defining the problem
</div>

<div class ="fragment">
<pre>
          system.parameters_filename = parameters_filename;
        
</pre>
</div>
<div class = "comment">
Construct a default RBTheta object, evaluate() returns 1.
</div>

<div class ="fragment">
<pre>
          RBTheta rb_theta;
        
</pre>
</div>
<div class = "comment">
Construct the assembly functors
</div>

<div class ="fragment">
<pre>
          Ex23DirichletDofAssembly dirichlet_assembly;
          ThetaA0 theta_a_0; A0 A0_assembly;
          ThetaA1 theta_a_1; A1 A1_assembly;
          ThetaA2 theta_a_2; A2 A2_assembly;
          F0 F0_assembly;
        
</pre>
</div>
<div class = "comment">
Define a set of output assembly functions. Each output
evaluates the solution average over a region defined by
the arguments to the OutputAssembly constructor
</div>

<div class ="fragment">
<pre>
          OutputAssembly L0(0.7,0.8,0.7,0.8);
          OutputAssembly L1(0.2,0.3,0.7,0.8);
          OutputAssembly L2(0.2,0.3,0.2,0.3);
          OutputAssembly L3(0.7,0.8,0.2,0.3);
        
          system.attach_dirichlet_dof_initialization(&dirichlet_assembly);
          
</pre>
</div>
<div class = "comment">
Attach the expansion of the PDE operator. The third argument
here refers to assembly over the boundary of the mesh, but this
problem only requires internal assembly and hence it is set to NULL
</div>

<div class ="fragment">
<pre>
          system.attach_A_q(&theta_a_0, &A0_assembly);
          system.attach_A_q(&theta_a_1, &A1_assembly);
          system.attach_A_q(&theta_a_2, &A2_assembly);
        
          
</pre>
</div>
<div class = "comment">
Attach the expansion of the RHS
</div>

<div class ="fragment">
<pre>
          system.attach_F_q(&rb_theta, &F0_assembly);
          
</pre>
</div>
<div class = "comment">
Attach output assembly
</div>

<div class ="fragment">
<pre>
          system.attach_output(&rb_theta, &L0);
          system.attach_output(&rb_theta, &L1);
          system.attach_output(&rb_theta, &L2);
          system.attach_output(&rb_theta, &L3);
          
</pre>
</div>
<div class = "comment">
We reuse the operator A0 as the inner product matrix
</div>

<div class ="fragment">
<pre>
          system.attach_inner_prod_assembly(&A0_assembly);
          
</pre>
</div>
<div class = "comment">
Initialize the RB data structures.
If we're in Offline Mode (online_mode == false) then
also pre-assemble the Dirichlet dofs list, RB operators etc
</div>

<div class ="fragment">
<pre>
          system.initialize_RB_system(online_mode);
        
          if(!online_mode)
          {
</pre>
</div>
<div class = "comment">
Compute the reduced basis space by computing "snapshots", i.e.
"truth" solves, at well-chosen parameter values and employing
these snapshots as basis functions.
</div>

<div class ="fragment">
<pre>
            system.train_reduced_basis();
            
</pre>
</div>
<div class = "comment">
Write out the reduced basis data
</div>

<div class ="fragment">
<pre>
            system.write_offline_data_to_files();
          }
          else
          {
</pre>
</div>
<div class = "comment">
Read in the reduced basis data
</div>

<div class ="fragment">
<pre>
            system.read_offline_data_from_files();
            
</pre>
</div>
<div class = "comment">
Get the parameters at which we do a reduced basis solve
</div>

<div class ="fragment">
<pre>
            unsigned int online_N = infile("online_N",1);
            std::vector&lt;Real&gt; online_mu_vector(system.get_n_params());
            for(unsigned int i=0; i&lt;system.get_n_params(); i++)
            {
              online_mu_vector[i] = infile("online_mu", online_mu_vector[i], i);
            }
        
</pre>
</div>
<div class = "comment">
Set the parameters to online_mu_vector
</div>

<div class ="fragment">
<pre>
            system.set_current_parameters(online_mu_vector);
            system.print_current_parameters();
        
</pre>
</div>
<div class = "comment">
Now do the Online solve using the precomputed reduced basis
</div>

<div class ="fragment">
<pre>
            system.rb_eval-&gt;RB_solve(online_N);
        
</pre>
</div>
<div class = "comment">
Print out outputs as well as the corresponding output error bounds.
</div>

<div class ="fragment">
<pre>
            std::cout &lt;&lt; "output 1, value = " &lt;&lt; system.rb_eval-&gt;RB_outputs[0]
                      &lt;&lt; ", bound = " &lt;&lt; system.rb_eval-&gt;RB_output_error_bounds[0]
                      &lt;&lt; std::endl;
            std::cout &lt;&lt; "output 2, value = " &lt;&lt; system.rb_eval-&gt;RB_outputs[1]
                      &lt;&lt; ", bound = " &lt;&lt; system.rb_eval-&gt;RB_output_error_bounds[1]
                      &lt;&lt; std::endl;
            std::cout &lt;&lt; "output 3, value = " &lt;&lt; system.rb_eval-&gt;RB_outputs[2]
                      &lt;&lt; ", bound = " &lt;&lt; system.rb_eval-&gt;RB_output_error_bounds[2]
                      &lt;&lt; std::endl;
            std::cout &lt;&lt; "output 4, value = " &lt;&lt; system.rb_eval-&gt;RB_outputs[3]
                      &lt;&lt; ", bound = " &lt;&lt; system.rb_eval-&gt;RB_output_error_bounds[3]
                      &lt;&lt; std::endl;
        
</pre>
</div>
<div class = "comment">
If we stored the basis functions in the offline_data directory, plot the RB solution
</div>

<div class ="fragment">
<pre>
            if(system.store_basis_functions)
            {
              system.load_RB_solution();
              ExodusII_IO(mesh).write_equation_systems ("RB_sol.exd",equation_systems);
              
              system.load_basis_function(0);
              ExodusII_IO(mesh).write_equation_systems ("bf0.exd",equation_systems);
            }
          }
        
          return 0;
        }
</pre>
</div>

<a name="nocomments"></a> 
<br><br><br> <h1> The program without comments: </h1> 
<pre> 
    
    
  
  #include &lt;iostream&gt;
  #include &lt;algorithm&gt;
  #include &lt;cmath&gt;
  #include &lt;set&gt;
  
  #include <B><FONT COLOR="#BC8F8F">&quot;libmesh.h&quot;</FONT></B>
  #include <B><FONT COLOR="#BC8F8F">&quot;mesh.h&quot;</FONT></B>
  #include <B><FONT COLOR="#BC8F8F">&quot;mesh_generation.h&quot;</FONT></B>
  #include <B><FONT COLOR="#BC8F8F">&quot;exodusII_io.h&quot;</FONT></B>
  #include <B><FONT COLOR="#BC8F8F">&quot;equation_systems.h&quot;</FONT></B>
  #include <B><FONT COLOR="#BC8F8F">&quot;dof_map.h&quot;</FONT></B>
  #include <B><FONT COLOR="#BC8F8F">&quot;getpot.h&quot;</FONT></B>
  #include <B><FONT COLOR="#BC8F8F">&quot;o_string_stream.h&quot;</FONT></B>
  #include <B><FONT COLOR="#BC8F8F">&quot;elem.h&quot;</FONT></B>
  
  #include <B><FONT COLOR="#BC8F8F">&quot;simple_rb.h&quot;</FONT></B>
  #include <B><FONT COLOR="#BC8F8F">&quot;assembly.h&quot;</FONT></B>
  
  
  
  
  <B><FONT COLOR="#228B22">int</FONT></B> main (<B><FONT COLOR="#228B22">int</FONT></B> argc, <B><FONT COLOR="#228B22">char</FONT></B>** argv)
  {
    LibMeshInit init (argc, argv);
  
  #<B><FONT COLOR="#A020F0">if</FONT></B> !defined(LIBMESH_HAVE_XDR)
    libmesh_example_assert(false, <B><FONT COLOR="#BC8F8F">&quot;--enable-xdr&quot;</FONT></B>);
  #elif defined(LIBMESH_DEFAULT_SINGLE_PRECISION)
    libmesh_example_assert(false, <B><FONT COLOR="#BC8F8F">&quot;--disable-singleprecision&quot;</FONT></B>);
  #endif
    libmesh_example_assert(libMesh::default_solver_package() == PETSC_SOLVERS, <B><FONT COLOR="#BC8F8F">&quot;--enable-petsc&quot;</FONT></B>);
  
    libmesh_example_assert(2 &lt;= LIBMESH_DIM, <B><FONT COLOR="#BC8F8F">&quot;2D support&quot;</FONT></B>);
    
    <B><FONT COLOR="#5F9EA0">std</FONT></B>::string parameters_filename = <B><FONT COLOR="#BC8F8F">&quot;ex23.in&quot;</FONT></B>;
    GetPot infile(parameters_filename);
  
    <B><FONT COLOR="#228B22">unsigned</FONT></B> <B><FONT COLOR="#228B22">int</FONT></B> n_elem = infile(<B><FONT COLOR="#BC8F8F">&quot;n_elem&quot;</FONT></B>, 1);       <I><FONT COLOR="#B22222">// Determines the number of elements in the &quot;truth&quot; mesh
</FONT></I>    <B><FONT COLOR="#228B22">const</FONT></B> <B><FONT COLOR="#228B22">unsigned</FONT></B> <B><FONT COLOR="#228B22">int</FONT></B> dim = 2;                      <I><FONT COLOR="#B22222">// The number of spatial dimensions
</FONT></I>    
    <B><FONT COLOR="#228B22">bool</FONT></B> online_mode = infile(<B><FONT COLOR="#BC8F8F">&quot;online_mode&quot;</FONT></B>, false); <I><FONT COLOR="#B22222">// Are we in Online mode?
</FONT></I>  
    Mesh mesh (dim);
    <B><FONT COLOR="#5F9EA0">MeshTools</FONT></B>::Generation::build_square (mesh,
                                         n_elem, n_elem,
                                         0., 1.,
                                         0., 1.,
                                         QUAD4);
  
    EquationSystems equation_systems (mesh);
  
    SimpleRB &amp; system =
      equation_systems.add_system&lt;SimpleRB&gt; (<B><FONT COLOR="#BC8F8F">&quot;RBConvectionDiffusion&quot;</FONT></B>);
  
  
    equation_systems.init ();
  
    <B><FONT COLOR="#A020F0">if</FONT></B>(system.initialize_mesh_dependent_data)
    {
      equation_systems.print_info();
      mesh.print_info();
    }
  
    
    system.parameters_filename = parameters_filename;
  
    RBTheta rb_theta;
  
    Ex23DirichletDofAssembly dirichlet_assembly;
    ThetaA0 theta_a_0; A0 A0_assembly;
    ThetaA1 theta_a_1; A1 A1_assembly;
    ThetaA2 theta_a_2; A2 A2_assembly;
    F0 F0_assembly;
  
    OutputAssembly L0(0.7,0.8,0.7,0.8);
    OutputAssembly L1(0.2,0.3,0.7,0.8);
    OutputAssembly L2(0.2,0.3,0.2,0.3);
    OutputAssembly L3(0.7,0.8,0.2,0.3);
  
    system.attach_dirichlet_dof_initialization(&amp;dirichlet_assembly);
    
    system.attach_A_q(&amp;theta_a_0, &amp;A0_assembly);
    system.attach_A_q(&amp;theta_a_1, &amp;A1_assembly);
    system.attach_A_q(&amp;theta_a_2, &amp;A2_assembly);
  
    
    system.attach_F_q(&amp;rb_theta, &amp;F0_assembly);
    
    system.attach_output(&amp;rb_theta, &amp;L0);
    system.attach_output(&amp;rb_theta, &amp;L1);
    system.attach_output(&amp;rb_theta, &amp;L2);
    system.attach_output(&amp;rb_theta, &amp;L3);
    
    system.attach_inner_prod_assembly(&amp;A0_assembly);
    
    system.initialize_RB_system(online_mode);
  
    <B><FONT COLOR="#A020F0">if</FONT></B>(!online_mode)
    {
      system.train_reduced_basis();
      
      system.write_offline_data_to_files();
    }
    <B><FONT COLOR="#A020F0">else</FONT></B>
    {
      system.read_offline_data_from_files();
      
      <B><FONT COLOR="#228B22">unsigned</FONT></B> <B><FONT COLOR="#228B22">int</FONT></B> online_N = infile(<B><FONT COLOR="#BC8F8F">&quot;online_N&quot;</FONT></B>,1);
      <B><FONT COLOR="#5F9EA0">std</FONT></B>::vector&lt;Real&gt; online_mu_vector(system.get_n_params());
      <B><FONT COLOR="#A020F0">for</FONT></B>(<B><FONT COLOR="#228B22">unsigned</FONT></B> <B><FONT COLOR="#228B22">int</FONT></B> i=0; i&lt;system.get_n_params(); i++)
      {
        online_mu_vector[i] = infile(<B><FONT COLOR="#BC8F8F">&quot;online_mu&quot;</FONT></B>, online_mu_vector[i], i);
      }
  
      system.set_current_parameters(online_mu_vector);
      system.print_current_parameters();
  
      system.rb_eval-&gt;RB_solve(online_N);
  
      <B><FONT COLOR="#5F9EA0">std</FONT></B>::cout &lt;&lt; <B><FONT COLOR="#BC8F8F">&quot;output 1, value = &quot;</FONT></B> &lt;&lt; system.rb_eval-&gt;RB_outputs[0]
                &lt;&lt; <B><FONT COLOR="#BC8F8F">&quot;, bound = &quot;</FONT></B> &lt;&lt; system.rb_eval-&gt;RB_output_error_bounds[0]
                &lt;&lt; std::endl;
      <B><FONT COLOR="#5F9EA0">std</FONT></B>::cout &lt;&lt; <B><FONT COLOR="#BC8F8F">&quot;output 2, value = &quot;</FONT></B> &lt;&lt; system.rb_eval-&gt;RB_outputs[1]
                &lt;&lt; <B><FONT COLOR="#BC8F8F">&quot;, bound = &quot;</FONT></B> &lt;&lt; system.rb_eval-&gt;RB_output_error_bounds[1]
                &lt;&lt; std::endl;
      <B><FONT COLOR="#5F9EA0">std</FONT></B>::cout &lt;&lt; <B><FONT COLOR="#BC8F8F">&quot;output 3, value = &quot;</FONT></B> &lt;&lt; system.rb_eval-&gt;RB_outputs[2]
                &lt;&lt; <B><FONT COLOR="#BC8F8F">&quot;, bound = &quot;</FONT></B> &lt;&lt; system.rb_eval-&gt;RB_output_error_bounds[2]
                &lt;&lt; std::endl;
      <B><FONT COLOR="#5F9EA0">std</FONT></B>::cout &lt;&lt; <B><FONT COLOR="#BC8F8F">&quot;output 4, value = &quot;</FONT></B> &lt;&lt; system.rb_eval-&gt;RB_outputs[3]
                &lt;&lt; <B><FONT COLOR="#BC8F8F">&quot;, bound = &quot;</FONT></B> &lt;&lt; system.rb_eval-&gt;RB_output_error_bounds[3]
                &lt;&lt; std::endl;
  
      <B><FONT COLOR="#A020F0">if</FONT></B>(system.store_basis_functions)
      {
        system.load_RB_solution();
        ExodusII_IO(mesh).write_equation_systems (<B><FONT COLOR="#BC8F8F">&quot;RB_sol.exd&quot;</FONT></B>,equation_systems);
        
        system.load_basis_function(0);
        ExodusII_IO(mesh).write_equation_systems (<B><FONT COLOR="#BC8F8F">&quot;bf0.exd&quot;</FONT></B>,equation_systems);
      }
    }
  
    <B><FONT COLOR="#A020F0">return</FONT></B> 0;
  }
</pre> 
<a name="output"></a> 
<br><br><br> <h1> The console output of the program: </h1> 
<pre>
Makefile:69: .depend: No such file or directory
Updated .depend
Compiling C++ (in optimized mode) ex23.C...
Linking ex23-opt...
***************************************************************
* Running  ./ex23-opt
***************************************************************
 
./ex23-opt: symbol lookup error: ./ex23-opt: undefined symbol: _ZTIN7libMesh5QBaseE
make[2]: *** [run] Error 127
rm ex23.i686-pc-linux-gnu.opt.o
</pre>
</div>
<?php make_footer() ?>
</body>
</html>
<?php if (0) { ?>
\#Local Variables:
\#mode: html
\#End:
<?php } ?>
